@@ -9,15 +9,20 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ *
+ * RCS: @(#) $Id: tclPort.h,v 1.15 2005/05/10 18:34:47 kennykb Exp $
  */
 
 #ifndef _TCLPORT
 #define _TCLPORT
 
+#ifdef HAVE_TCL_CONFIG_H
+#include "tclConfig.h"
+#endif
 #include "tcl.h"
 
 #if defined(__WIN32__)
-#   include "tclWinPort.h"
+#   include "../win/tclWinPort.h"
 #else
 #   include "tclUnixPort.h"
 #endif
