@@ -33,7 +33,7 @@
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclStringObj.c,v 1.68 2007/12/13 15:23:20 dgp Exp $ */
+ * RCS: @(#) $Id: tclStringObj.c,v 1.70.2.1 2008/04/07 16:07:00 dgp Exp $ */
 
 #include "tclInt.h"
 #include "tommath.h"
@@ -2003,8 +2003,8 @@ Tcl_AppendFormatToObj(
 	    allocSegment = 1;
 	    Tcl_IncrRefCount(segment);
 
-	    if ((isNegative || gotPlus) && (useBig || (ch == 'd'))) {
-		Tcl_AppendToObj(segment, (isNegative ? "-" : "+"), 1);
+	    if ((isNegative || gotPlus || gotSpace) && (useBig || (ch == 'd'))) {
+		Tcl_AppendToObj(segment, (isNegative ? "-" : gotPlus ? "+" : " "), 1);
 	    }
 
 	    if (gotHash) {
@@ -2105,7 +2105,7 @@ Tcl_AppendFormatToObj(
 			numDigits++;
 			uw /= base;
 		    }
-		} else if (useBig) {
+		} else if (useBig && big.used) {
 		    int leftover = (big.used * DIGIT_BIT) % numBits;
 		    mp_digit mask = (~(mp_digit)0) << (DIGIT_BIT-leftover);
 
@@ -2114,7 +2114,7 @@ Tcl_AppendFormatToObj(
 			numDigits--;
 			mask >>= numBits;
 		    }
-		} else {
+		} else if (!useBig) {
 		    unsigned long int ul = (unsigned long int) l;
 
 		    bits = (Tcl_WideUInt) ul;
@@ -2138,7 +2138,7 @@ Tcl_AppendFormatToObj(
 		while (numDigits--) {
 		    int digitOffset;
 
-		    if (useBig) {
+		    if (useBig && big.used) {
 			if ((size_t) shift <
 				CHAR_BIT*sizeof(Tcl_WideUInt) - DIGIT_BIT) {
 			    bits |= (((Tcl_WideUInt)big.dp[index++]) <<shift);
@@ -2154,6 +2154,9 @@ Tcl_AppendFormatToObj(
 		    }
 		    bits /= base;
 		}
+		if (useBig) {
+		    mp_clear(&big);
+		}
 		if (gotPrecision) {
 		    while (length < precision) {
 			Tcl_AppendToObj(segment, "0", 1);
