@@ -9,11 +9,11 @@
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+# 
+# RCS: @(#) $Id: all.tcl,v 1.19 2006/11/03 00:34:52 hobbs Exp $
 
-set tcltestVersion [package require tcltest]
-namespace import -force tcltest::*
-
-tcltest::testsDirectory [file dir [info script]]
-tcltest::runAllTests
-
-return
+package require Tcl 8.5
+package require tcltest 2.2
+namespace import tcltest::*
+configure {*}$argv -testdir [file dir [info script]]
+runAllTests
